{% extends 'base.html' %}

{% block title%}Airbyte Connector Registry{% endblock %}

{% block content %}
<h1>Airbyte Connector Registries</h1>
<h2>Sources</h2>
<div class="table-container test">
  {{ sources_table_html }}
</div>

<h2>Destinations</h2>
<div class="table-container">
  {{ destinations_table_html }}
</div>
{% endblock %}